//
//  Guit GUI Toolkit Examples.
//  Copyright © 2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <guit.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;


class Bindings : public GWindow {
public:
  Bindings() : GWindow("#bindings Bindings") {
    *this
    << (Table("{align: c c; gap:8 8; flex:0 0}")
        << (TRow
            << shared_value()
            << calculations()
            )
        << (TRow
            << euros_to_dollars()
            << meters_to_imperial()
            )
        << (TRow
            << dynamic_text()
            << colored_text()
            )
        << (TRow
            << colors()
            << shared_props()
            )
        );
  }

private:

  GBox& shared_value() {
    auto& val = Float(0.f, -100.f, +100.f);

    return VBox(".panel .bigtext Shared Value")
    << (Table("{align: s x; gap:7 5}")
        << (TRow() << "<b>Label"        << (Label << val))
        << (TRow() << "<b>TextField:"   << (TextField << val))
        << (TRow() << "<b>TextField#2:" << val)   // creates a TextField by default
        << (TRow() << "<b>Slider:"      << (Slider << val))
        );
  }


  GBox& calculations() {
    auto &a = Float(), &b = Float(), &c = Float(), &d = Float();

    return Table(".panel .smalltext Calculations")
    << (TRow()
        << a << "+" << b << " = " << (~a + ~b)
        )
    << (TRow()
        << c << "x" << d << " = " << (~c * ~d)
        )
    << (TRow()
        << a << "+" << b << " = " << c << "x" << d
        << " is" << (~a + ~b == ~c * ~d)
        );
  }


  GBox& euros_to_dollars() {
    auto& euros = Float(100);
    auto& dollars = Float();
    auto& rate = Float(1.12f);

    dollars <<= ~euros * ~rate;    // constraints
    euros   <<= ~dollars / ~rate;

    return Table(".panel Euros to Dollars")
    << ".bigtitle Shared Value"
    << (TRow()
        << "Euros:"
        << euros
        << "   "
        << "Rate:"
        << rate
        )
    << (TRow()
        << "Dollars:"
        << dollars
        );
    }


  GBox& meters_to_imperial() {
    static std::vector<float> conv {   // not copied => static
      1.f,         // m => m
      100.f,       // m => cm
      0.0001f,     // m => km
      3.28084f,    // m => foot
      1.09361296f, // m => yard
      1.f/1609.344f // m => mile
    };

    auto& dist = Float(1);
    auto& unit = Choice(0);
    auto& option = OptionBox("{items:m, cm,km,foot,yard,mile}") << unit;

    return VBox(".panel Meters to Imperial")
        << "<b>Meters to Choice:"
        << (HBox() << dist << "m" << "  equals" << ((~conv)[~unit] * ~dist) << option)
        ;
  }


  GBox& dynamic_text() {
    auto& character = Text("Columbo");
    auto& actor     = Text("Peter Falk");
    
    return Table(".panel .bigtext Dynamic Text")
    << (TRow() << "<b>Character:" << character)
    << (TRow() << "<b>Actor:"     << actor)
    << ("The role of <c:red><b>" + ~character + "</b></c> is played by <c:blue><b>" + ~actor);
  }


  GBox& colored_text() {
    auto& name1  = Text("Columbo");
    auto& name2  = Text("Peter Falk");
    auto& color1 = Text("red");
    auto& color2 = Text("blue");

    return Table(".panel .bigtext  Colored Text")
    << (TRow() << name1 << color1)
    << (TRow() << name2 << color2)
    << ("The color of <c:" + ~color1 + ">" + ~name1 + " </c> is <b>" + ~color1)
    << ("The color of <c:" + ~color2 + ">" + ~name2 + " </c> is <b>" + ~color2);
  }


  GBox& colors() {
    auto& red   = Float(1.f,  0.f, 1.f);
    auto& green = Float(0.5f, 0.f, 1.f);
    auto& blue  = Float(0.f,  0.f, 1.f);
    auto& alpha = Float(0.5f, 0.f, 1.f);

    return HBox(".panel Colors")
    << (Label("{size: 100 100; flex: 0 0}")
        << (Background() <<= ~red + " " + ~green + " " + ~blue + " " + ~alpha)
        )
    << (Table()
        << (TRow() << "Red:"   << (Slider << red))
        << (TRow() << "Green:" << (Slider << green))
        << (TRow() << "Blue:"  << (Slider << blue))
        << (TRow() << "Alpha:" << (Slider << alpha))
        << (Label("{format:2; margin:3 5; background:black; color:white; align: c e}")
                  << "RGBA: ("+ ~red +", " + ~green +", "+ ~blue +", "+ ~alpha +")"
            )
        );
  }


  GBox& shared_props() {
    auto& check  = Check(true);
    auto& enable = Enable(true);

    return VBox(".panel Dependent Props")
    << (CheckItem
        << check
        << enable
        << on::checked / "Button is checked"
        << on::unchecked  / "Button is unchecked"
        )
    << (HBox
        << "Text:"s
        << (TextField("{size:25c a} blah blah") << enable)
        )
    << (HBox("{flex:1 0}")
        << "Ckeck:"s
        << (Button
            << check
            << on::checked / "yes"
            << on::unchecked / "no" / GBackground::orange / GColor::white
            )
        << "Enable:"s
        << (Button
            << Check(true)
            << on::checked / "yes" / [&]{enable = true;}
            << on::unchecked / "no" / GBackground::orange / GColor::white / [&]{enable = false;}
            )
        );
  }

};


GWindow& Examples::makeBindings() {
  return *new Bindings();
}
