//
//  Guit GUI Toolkit Examples.
//  Copyright © 2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <guit.hpp>
#include <ginspector.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;

int main(int argc, const char *argv[]) {
  GApp app(argc, argv);
  app.inspector();

  // special case: creates a client app
  if (argc >= 2 && argv[1] == "-client"s) {
    app << "remote-app.css";
    return app.start(Examples::makeClientApp());
  }

  app
  << "examples.css"
  << "boxes-bars.css"
  << "one-page.css"
  << "multi-pages.css"
  << "bindings.css"
  << "text.css"
  << "forms.css"
  << "multiview-table.css"
  << "charts.css"
  << "animations.css"
  << "threads.css"
  << "opengl.css"
  << "clock.css"
  << "traffic-lights.css"
  << "control-panel.css"
  << "microwave.css"
  << "pies-grids.css"
  << "tictactoe.css"
  << "diagram.css"
  << "drawing.css"
  << "misc.css"
  << "styles.css"
  << "remote-app.css";

  return app.start(new Examples);
};


Examples::Examples() : GWindow("#examples Guit Examples") {
  *this
  // the QuitDialog will be opened when the application is closed
  // << QuitDialog("Quit App")
  << (HBox()
      << (ButtonBox()
          << ("Boxes & Bars"    << makeBoxesAndBars())
          << ("One Page"        << makeOnePage())
          << ("Multi-Pages"     << makeMultiPages())
          << ("Bindings"        << makeBindings())
          << ("Traffic Lights"  << makeTrafficLights())
          << ("Basic Text"      << makeBasicText())
          << ("Attributed Text" << makeAttributedText())
          << ("Simple Form"     << makeSimpleForm())
          << ("Multiview Table" << makeMultiViewTable())
          << ("Charts"          << makeCharts())
          << ("Pies & Grids"    << makePiesAndGrids())
          << ("Animations"      << makeAnimations())
          )
      << (ButtonBox()
          << ("Clock"           << makeClock())
          << ("Microwave Oven"  << makeMicrowave())
          << ("OpenGL"          << makeOpenGL())
          << ("Drawing"         << makeDrawing())
          << ("Diagram"         << makeDiagram())
          << ("Control Panel"   << makeControlPanel())
          << ("TicTacToe"       << makeTicTacToe())
          << ("Threads"         << makeThreads())
          << ("Remote Apps"     << makeServerApp())
          << ("Misc"            << makeMisc())
          << ("Styles"          << makeStyles())
          )
      )
  << (StatusBar
      << (Button("Quit") << []{GApp::close();})
      << Filler
      << (Button("Inspector") << []{ GApp::showInspector(); })
      );
}

