//
//  Simple Form
//  Guit GUI Toolkit
//  Copyright © 2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <sstream>
#include <guit.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;

class SimpleForm : public GWindow {
  GRegText firstname, lastname;  // constrained text
  GChoice gender;
  GInt age;
  GRegText phone;
  GText email;
  GText userlist;
  GString alert = "Must be filled in!";

public:
  SimpleForm() : GWindow("#simple-form Simple Form") {
    firstname.setNameFormat();
    lastname.setNameFormat();
    phone.setPhoneFormat();

    // ** create the form
    // field names can be changed in CSS.
    // appropriate gadgets are automatically created and bound to the props
    auto& form = Table("#form")
    << (TRow() << "#fname"  << firstname)
    << (TRow() << "#lname"  << lastname)
    << (TRow() << "#gender" << (OptionBox("#genders") <<gender))
    << (TRow() << "#age"    << age)
    << (TRow() << "#phone"  << phone)
    << (TRow() << "#email"  << email);

    // ** create the vertical controls
    auto& buttons = ButtonBox("#buttonbox")
    << ("#next"
        // blink these fields if there are empty:
        << Guard(~firstname == "") / [&]{ firstname.blink(alert); }
        << Guard(~lastname == "")  / [&]{ lastname.blink(alert); }
        << Guard(~gender < 0)      / [&]{ gender.blink(alert); }
        // otherwise, if all previous clauses are false
        << Otherwise() / [&]{doNext(form);}
        )
    << ("#clear" << [&]{form.clearValues();})        // clears the form
    << ("#show"                 // opens a dialog showing entered users
        << (Dialog("#userdial User List")
            << (ScrollBox << (TextArea << userlist))
            )
        );

    // ** add gadgets to window
    *this
    << QuitDialog("Close Window")
    << "#form_header"
    << (HBox("#form_panel") << form << " " << buttons);
  }


  void doNext(GTable& form) {
    form.write(cout, GIOMode::Values);  // prints all values

    ostringstream ss;
    form.write(ss, GIOMode::Values);
    userlist += ss.str();
    cerr << "userlist: " << userlist << endl;
    form.clearValues();
  }
};


GWindow& Examples::makeSimpleForm() {
  return *new SimpleForm;
}
