//
//  text example
//  Guit GUI Toolkit
//  Copyright © 2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <guit.hpp>
#include "examples.hpp"
using namespace std;
using namespace guit;


class BasicText : public GWindow {
  GTitle title{"Basic Text"};  // window title (changed when a new file is read)
  GText text;                  // displayed text
  GBool changed;               // set to true when text is changed

public:
  BasicText() : GWindow("#basic-text Basic Text")
  {
    // reads this file
    text.read(":data/lorem-ipsum.txt");

    // changed set to true when text's value is changed
    text << [&]{changed = true;};
    changed = false;

    // creates dialog for opening a file
    // ":" is the resource directory
    // alertIf() : shows an alert before opening the dialog if changed is true
    auto& opendial = OpenDialog("Open File",":/data").alertIf(changed);
    opendial << [&] {
      text.read(opendial.pathname());
      *this << Title("Basic Text: " + basename(opendial.pathname()));
      changed = false;
    };

    // creates dialog for saving the text
    // "~" is the home directory of the user
    auto& savedial = SaveDialog("Save File", "~");
    savedial << [&] {
      text.write(savedial.pathname());
      changed = false;
    };

    // create Quit Dialog
    // closes the window this dialog is added to
    // openOnlyIf() : show the dialog only if changed is true
    auto& closedial =
    QuitDialog("Close", "Text has changed\ndo you want to close this window?")
    .openOnlyIf(changed);

    // add gadgets to window
    *this
    << title
    << closedial    // opened before attenmpting to close the window
    << (ScrollBox << (TextArea << text))   // scrollbox containing the text
    << (HBox("#controls")                   // contains commands
        << (Button("Open") << opendial)
        << (Button("Save") << savedial)
        << (Button("Close") << closedial)
        );
  }
};



class AttributedText : public GWindow {
  GText text;
  GBool changed;

public:
  AttributedText() : GWindow("#attr-text Attributed Text")
  {
    text.read(":data/blue-orange.txt");
    text << [this]{changed = true;};
    changed = false;

    auto& opendial = OpenDialog("Open File",":/data").alertIf(changed);
    opendial << [&] {
      text.read(opendial.pathname());
      *this << Title("Attributed Text: " + basename(opendial.pathname()));
      changed = false;
    };

    auto& savedial = SaveDialog("Save File", "~");
    savedial << [&] {
      text.write(opendial.pathname());
      changed = false;
    };

    auto& closedial =
    QuitDialog("Close", "Text has changed\ndo you want to close this window?")
    .openOnlyIf(changed);

    *this
    << closedial
    << (VBox(".tf-panel Labels")
        << ".subtitle Unformatted:"
        << (Label("{border:squared}") << text).setRichText(false)
        << ".subtitle Formatted (default):"
        << (Label("{border:squared}") << text)
        )
    << (VBox(".tf-panel TextFields")
        << ".subtitle Unformatted (default):"
        << (TextField << text)
        << ".subtitle Formatted:"
        << (TextField << text).setRichText(true)
        )
    << (VBox(".ta-panel TextAreas")
        << ".subtitle Unformatted (default):"
        << (ScrollBox << (TextArea << text))
        << ".subtitle Formatted:"
        << (ScrollBox << (TextArea << text).setRichText(true))
        )
    << (HBox("#controls")
        << (Button("Open") << opendial)
        << (Button("Save") << savedial)
        << (Button("Close") << closedial)
        );
  }
};


GWindow& Examples::makeBasicText() {
  return *new BasicText();
}

GWindow& Examples::makeAttributedText() {
return *new AttributedText();
}
