//
//  Traffic light with state machine
//  Guit GUI Toolkit
//  Copyright © 2020 Eric Lecolinet. All rights reserved.
//  http://www.telecom-paris.fr/~elc
//

#include <guit.hpp>
#include <gshape.hpp>
#include "examples.hpp"
using namespace guit;
using namespace std;

class TrafficLights : public GWindow {
public:
  TrafficLights() : GWindow("#traffic-lights Traffic Lights")
  {
    auto& timer = Timer(2000);
    auto& beat = Int();
    timer.addBeat(beat, 1, 3);

    auto& lights = Box("#lights")
    << (OvalShape() << (~beat == 0) / GBackground::green)
    << (OvalShape() << (~beat == 1) / GBackground::orange)
    << (OvalShape() << (~beat == 2) / GBackground::red);

    auto& light_name = Label("#light-name")
    << (~beat == 0) / "Go" / GColor::green  
    << (~beat == 1) / "Wait" / GColor::orange
    << (~beat == 2) / "Stop" / GColor::red;

    // adds gadgets to the window
    *this
    << lights
    << light_name
    << (Button("Next") << [&]{beat++;})
    << (Button("Anim").setCheckable()           // starts/stops the animation
        << on::checked   / [&]{timer.start();}
        << on::unchecked / [&]{timer.stop();}
        );
  }
};


GWindow& Examples::makeTrafficLights() {
  return *new TrafficLights();
}
