//
//  Implementation of the Interaction Controller
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef Guit_ControlImpl_hpp
#define Guit_ControlImpl_hpp
#include <list>
#include <gcontrol.hpp>
namespace guit {

/// GUI Controller.
class GControlImpl : public GCore {
public:
  enum {TimerEvent, FunEvent};

  static GControlImpl& instance();
  GControlImpl();
  ~GControlImpl();

private:
  friend class GNatContext;
  friend class GNatWindow;

  bool doInspect(GWindow*, Gadget* obj, GMouseEvent&, GPoint const& pos);
  bool checkModal(GWindow*);
  void funCallDone();
  void paintWindow(GWindow*);

  void mouseDown(GMouseEvent&);
  void mouseUp(GMouseEvent&);
  void mouseMove(GMouseEvent&);
  void wheelMove(GMouseEvent&);
  void keyDown(GKeyEvent&);
  void keyUp(GKeyEvent&);
  void keyType(GKeyEvent&);
  void touchEvent(GTouchEvent&);
  void gestureEvent(GGestureEvent&);
  void windowShown(GWindow&, GTime);
  void windowHidden(GWindow&, GTime);
  void windowResized(GWindow&, GTime, float w, float h);
  void windowMoved(GWindow&, GTime, float x, float y);
  void windowMaximized(GWindow&, GTime);
  void windowMinimized(GWindow&, GTime);
  void windowClosed(GWindow&, GTime);
  void windowEntered(GWindow&, GTime);
  void windowExited(GWindow&, GTime);
  void windowfocusIn(GWindow&, GTime);
  void windowfocusOut(GWindow&, GTime);
};

}
#endif
