//
// Gadget IO
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
#include <fstream>
#include <cstring>
#include <cstdio>
#include <sys/types.h>
#include <sys/stat.h>
#include <gstring.hpp>
#include <gfile.hpp>
#include <gapp.hpp>
#include <gcontrol.hpp>
#include <giobuffer.hpp>
namespace guit {

GIOBuffer::GIOBuffer(std::istream& in) :
level_(-1), in_(&in) {}

GIOBuffer::GIOBuffer(std::ostream& out) :
level_(-1), out_(&out) {}

void GIOBuffer::setValueMode(bool val) {
  valuemode_ = val;
  if (out_) level_ = val ? -1 : 0;
}

void GIOBuffer::clear() {
  hasdata_ = false;
  data_.clear();
}

bool GIOBuffer::read() {
  if (hasdata_) return true;
  tabcount_ = 0;
  clear();
  bool start = true;
  char c = 0;
  int spaces = 0;

  while (in_->get(c)) {
    if (start && c == ' ') {  // beginning of line => count spaces
      spaces++;
    }
    else if (start && c == '#') {  // line starting by # => comment
      if (begin_) {
        begin_ = false;
        data_ += c;
        while (in_->get(c) && c != '\n') data_ += c;
        return true;
      }
      else {
        while (in_->get(c) && c != '\n') {}
      }
    }
    else if (c == '\n' || (valuemode_ && c == ';')) {
      if (!hasdata_) {
        tabcount_ = spaces / 2;
        if (spaces % 2 != 0) data_ += ' ';
      }
      hasdata_ = true;
      return true;
    }
    else {
      hasdata_ = true;
      tabcount_ = spaces / 2;
      if (spaces % 2 != 0) data_ += ' ';   // bug ???
      data_ += c;
      start = false;
      begin_ = false;
    }
  }
  if (in_->eof()) return false;
  else if (in_->fail()) {
    gerror("Gadget::read", "reading error " + data());
    throw IOError();
  }
  return false;
}

void GIOBuffer::printTabbed(GString const& token) {
  for (int k = 0; k < level_; ++k) {
    *out_ << "  ";  // yaml tab = 2 spaces
  }
  *out_ << token;
}

void GIOBuffer::printDelim() {
  *out_ << (insiderow_ ? ';' : '\n');
}

void GIOBuffer::setInsideRow(bool val) {
  insiderow_ = val;
}

}
