//
//  Spin Item
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef Guit_SpinItem_hpp
#define Guit_SpinItem_hpp
#include <gtextitem.hpp>
namespace guit {

struct GSpinItem : public GTextItem {
protected:
  GSpinItem(GadgetType&, GString const& css_and_value);

public:
  ~GSpinItem();
  GSpinItem(GString const& css_and_value = "");
  GSpinItem& clone(GClone const&) override;

  using GTextItem::operator=;
  GTextItem& operator=(float) override;

  /// Returns the value as a number.
  /// Returns the value of the associated GNumber prop (see operator()).
  /// The value lies between min() and max() (0 and 100 by default)
  /// @{
  int intValue() const override;
  float floatValue() const override;
  /// @}

  /// Returns/changes the minimum and maximum values.
  /// Returns/changes the min and max value of the associated GNumber prop (see operator()).
  /// The default min is 0, the default max is 100.
  /// @{
  GSpinItem& setRange(float min, float max);
  float min() const;
  float max() const;
  /// @}

  /// Returns the prop that controls the value.
  /// The prop that controls the value can be changed by using the << operator:
  /// @code
  ///    auto& value = Float();
  ///    auto& item = ValueItem() << value;
  ///    value = 10.f;
  /// @endcode
  /// @{
  GNumber& numProp();
  /// @}

  using GType = GadgetType_<GSpinItem,GTextItem>;
  static GType& Type();
  GType& type() const override {return Type();}

  /// @internal
  /// @{
  void addProp(GProp& prop, bool add_to_list) override;
  void mouseCB(GMouseEvent&);
  gptr<GNumber> numvalue_; 
  float incr_{1.f};
  gptr<class GIcon> icon_;
  /// @}
};

GSpinItem& SpinItem(GString const& css_and_value = "");

}
#endif
