//
// Gap between the children of a gadget
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef Guit_Gap_hpp
#define Guit_Gap_hpp
#include <gvarprop.hpp>
namespace guit {

class GGapValue {
public:
  GGapValue() = default;
  GGapValue(float main_spacing, float cross_spacing);
  bool operator==(GGapValue const&) const;
  bool operator!=(GGapValue const&) const;
  float mainspacing{1.f}, crossspacing{1.f};
};

inline bool gconvert(GGapValue const& from, GGapValue& to) {to = from; return true;}
bool gconvert(GString const& from, GGapValue& to);
bool gconvert(GGapValue const& from, GString& to);


/// Gap between gadgets.
class GGap : public GVarProp_<GGapValue> {
public:
  using Super = GVarProp_<GGapValue>;

  GGap();
  GGap(GString const& value);
  GGap(float main_spacing, float croos_spacing);
  GGap(GGap const& GGap);
  GGap(GGapValue const&);

  GGap* clone(bool copy_value) const override;

  using GType = GPropType_<GGap,GProp>;
  static GType& Type();
  GType& type() const override {return Type();}

  void updateGadget(Gadget&) override;
  void apply(Gadget*, GRender&, Specif) final;
};

/// Returns a GGap created in the heap.
/// @{
inline GGap& Gap() {return *new GGap();}
inline GGap& Gap(GString const& value) {return *new GGap(value);}
/// @}

}
#endif
