//
// CSS internals: Value and Item props
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gevent.hpp>
#include <gvalue.hpp>
#include <gbox.hpp>
#include <gcontrol.hpp>
#include <grender.hpp>
#include <gstring.hpp>
using namespace std;
namespace guit {

GIMPLEMENT_PROP_TYPE(Items,GPropType::Auto)
GIMPLEMENT_PROP_TYPE(Value,GPropType::Auto)


bool GValue::onAdd(Gadget* g) {
  if (!g) return true;
  if (!g->hasValue()) {
    error("onAdd","Can't add a 'Value' prop to a '"+ g->typeName()+"' gadget");
    return false;
  }
  else {
    g->setValue(value_);     // faire un bind ??
    return true;
  }
}

GItems::GItems(GString const& items) : items_(items) {
  // search ad hoc class for all items of this box
  size_t from{};
  itemclass_ = parseClass(items, from);
}

// used by direct props & inline styles (not CSS styles)
bool GItems::onAdd(Gadget* g) {
  if (!g) return true;
  if (auto* box = g->toBox()) {
    //cerr << "GItems::add: "<< g->typeName() <<" " << this << endl;
    box->settings_.hasItems = 1;
    return true;
  }
  else {
    error("onAdd","'Items' prop can only be added to a GBox (or subclass)"
          "but this gadget is a " + g->typeName());
    return false;
  }
}

// used by direct props / inline styles / CSS styles
void GItems::apply(Gadget* g, GRender& r, Specif specif) {
  if (!g || g->settings_.itemsSet) return;  // alraedy done, dont do it again
  if (auto* box = g->toBox()) {
    box->settings_.hasItems = 1;  // needed for CSS styles
    box->settings_.itemsSet = 1;  // dont do it again
    //cerr << "GItems::apply: "<< g->typeName() <<" " << this << " items: "<<items_ << endl;
    //box->addItems(items_);
    GString defclass;
    size_t pos{};
    GString it;
    while (strtoken(items_, it, pos, ",") != -1) {
      box->addItemImpl(it, -1, defclass);
    }
  }
}
/*
void GBox::addItems(GString const& items) {
  GString defclass;
  size_t pos{};
  GString it;
  while (strtoken(items, it, pos, ",") != -1) addItemImpl(it, -1, defclass);
}*/

GString GItems::parseClass(GString const& str, size_t& pos) {
  size_t len = str.size();

  while (pos < len && ::isspace(str[pos])) pos++;
  if (pos >= len || str[pos] != '@') {
    pos = 0;
    return "";
  }
  pos++;

  GString cname;
  int delim = strtoken(str, cname, pos, "{ \t\n");
  if (delim == -1 || cname.empty()) {
    pos = 0;
    return "";
  }
  //cerr << "cname " << cname << endl;
  if (delim == '{') pos--;
  return cname;
}

}
